"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class Events {
}
exports.Events = Events;
Events.QueueManager_InterruptJob = "QueueManager_InterruptJob";
Events.QueueManager_InterruptJobSuccess = jobId => "QueueManager_InterruptJobSuccess_" + jobId;
Events.QueueManager_JobExecuted = "QueueManager_JobExecuted";
class EventBus {
    constructor() {
        this.listeners = {};
    }
    on(eventName, listener) {
        this.addListener(eventName, listener, false);
    }
    once(eventName, listener) {
        this.addListener(eventName, listener, true);
    }
    addListener(eventName, listener, once) {
        if (typeof listener != "function") {
            return;
        }
        let eventListeners = this.listeners[eventName];
        if (!eventListeners) {
            this.listeners[eventName] = eventListeners = [];
        }
        const index = eventListeners.indexOf(listener);
        if (index == -1) {
            eventListeners.push(listener);
        }
        listener.$once = once;
    }
    removeListener(eventName, listener) {
        const eventListeners = this.listeners[eventName];
        if (eventListeners) {
            const index = eventListeners.indexOf(listener);
            if (index > -1) {
                eventListeners.splice(index, 1);
            }
        }
    }
    removeAllListeners(eventName) {
        if (eventName == null) {
            this.listeners = {};
        }
        else {
            delete this.listeners[eventName];
        }
    }
    emit(eventName, ...args) {
        const promises = [];
        const eventListeners = this.listeners[eventName];
        if (eventListeners) {
            const rm = [];
            for (let i = 0; i < eventListeners.length; i++) {
                const eventListener = eventListeners[i];
                promises.push(eventListener(...args));
                if (eventListener.$once) {
                    rm.push(i);
                }
            }
            if (rm.length) {
                for (let i = rm.length - 1; i > -1; i--) {
                    eventListeners.splice(i, 1);
                }
            }
        }
        return Promise.all(promises);
    }
}
exports.EventBus = EventBus;
//# sourceMappingURL=Events.js.map